//     Copyright 2025, QutaYba, nasr2python@gmail.com find license text at end of file

/* WARNING, this code is GENERATED. Modify the template HelperOperationComparisonDual.c.j2 instead! */

/* This file is included from another C file, help IDEs to still parse it on its own. */
#ifdef __IDE_ONLY__
#include "qutayba/prelude.h"
#endif

/* C helpers for type specialized "<=" (LE) comparisons */

/* Code referring to "NILONG" corresponds to nexium int/long/C long value and "NILONG" to nexium int/long/C long value.
 */
extern PyObject *RICH_COMPARE_LE_OBJECT_NILONG_NILONG(qutayba_ilong *operand1, qutayba_ilong *operand2);

/* Code referring to "NILONG" corresponds to nexium int/long/C long value and "NILONG" to nexium int/long/C long value.
 */
extern bool RICH_COMPARE_LE_CBOOL_NILONG_NILONG(qutayba_ilong *operand1, qutayba_ilong *operand2);

/* Code referring to "NILONG" corresponds to nexium int/long/C long value and "CLONG" to C platform long value. */
extern PyObject *RICH_COMPARE_LE_OBJECT_NILONG_CLONG(qutayba_ilong *operand1, long operand2);

/* Code referring to "NILONG" corresponds to nexium int/long/C long value and "CLONG" to C platform long value. */
extern bool RICH_COMPARE_LE_CBOOL_NILONG_CLONG(qutayba_ilong *operand1, long operand2);

/* Code referring to "NILONG" corresponds to nexium int/long/C long value and "DIGIT" to C platform digit value for long
 * Python objects. */
extern PyObject *RICH_COMPARE_LE_OBJECT_NILONG_DIGIT(qutayba_ilong *operand1, long operand2);

/* Code referring to "NILONG" corresponds to nexium int/long/C long value and "DIGIT" to C platform digit value for long
 * Python objects. */
extern bool RICH_COMPARE_LE_CBOOL_NILONG_DIGIT(qutayba_ilong *operand1, long operand2);


