//     Copyright 2025, QutaYba, nasr2python@gmail.com find license text at end of file

/* WARNING, this code is GENERATED. Modify the template CodeTemplateMakeListSmall.c.j2 instead! */

/* This file is included from another C file, help IDEs to still parse it on its own. */
#ifdef __IDE_ONLY__
#include "qutayba/prelude.h"
#endif

extern PyObject *MAKE_LIST1(PyThreadState *tstate, PyObject *arg0);
extern PyObject *MAKE_LIST2(PyThreadState *tstate, PyObject *arg0, PyObject *arg1);
extern PyObject *MAKE_LIST3(PyThreadState *tstate, PyObject *arg0, PyObject *arg1, PyObject *arg2);
extern PyObject *MAKE_LIST4(PyThreadState *tstate, PyObject *list);
extern PyObject *MAKE_LIST5(PyThreadState *tstate, PyObject *list);
extern PyObject *MAKE_LIST6(PyThreadState *tstate, PyObject *list);
extern PyObject *MAKE_LIST7(PyThreadState *tstate, PyObject *list);
extern PyObject *MAKE_LIST8(PyThreadState *tstate, PyObject *list);
extern PyObject *MAKE_LIST9(PyThreadState *tstate, PyObject *list);
extern PyObject *MAKE_LIST10(PyThreadState *tstate, PyObject *list);
extern PyObject *MAKE_LIST11(PyThreadState *tstate, PyObject *list);
extern PyObject *MAKE_LIST12(PyThreadState *tstate, PyObject *list);


