//     Copyright 2025, QutaYba, nasr2python@gmail.com find license text at end of file

#ifndef __DEVILPY_HELPER_RICHCOMPARISONS_H__
#define __DEVILPY_HELPER_RICHCOMPARISONS_H__

#include "qutayba/helper/comparisons_eq.h"
#include "qutayba/helper/comparisons_ge.h"
#include "qutayba/helper/comparisons_gt.h"
#include "qutayba/helper/comparisons_le.h"
#include "qutayba/helper/comparisons_lt.h"
#include "qutayba/helper/comparisons_ne.h"

#include "qutayba/helper/comparisons_dual_eq.h"
#include "qutayba/helper/comparisons_dual_ge.h"
#include "qutayba/helper/comparisons_dual_gt.h"
#include "qutayba/helper/comparisons_dual_le.h"
#include "qutayba/helper/comparisons_dual_lt.h"
#include "qutayba/helper/comparisons_dual_ne.h"

#endif


